// -*- c++ -*-
#ifndef DEFAULT_MAP_H
#define DEFAULT_MAP_H


#include <hugo/array_map.h>
#include <hugo/vector_map.h>

///\ingroup graphmaps
///\file
///\brief Graph maps that construates and destruates
///their elements dynamically.

namespace hugo {

/// \addtogroup graphmaps
/// @{

  /** The ArrayMap template class is graph map structure what
   *  automatically updates the map when a key is added to or erased from
   *  the map. This map uses the VectorMap if the ValueType is a primitive
   *  type and the ArrayMap for the other cases.
   *
   *  The template parameter is the MapRegistry that the maps
   *  will belong to and the ValueType.
   */


  /** Macro to implement the DefaultMap.
   */
#define DEFAULT_MAP_BODY(DynMap, Value) \
  { \
    typedef DynMap<MapRegistry, Value> MapImpl; \
  \
  public: \
  \
    typedef typename MapRegistry::Graph Graph; \
  \
    DefaultMap() : MapImpl() {} \
  \
    DefaultMap(const Graph& g, MapRegistry& r) : MapImpl(g, r) {} \
  \
    DefaultMap(const Graph& g, MapRegistry& r, const Value& v) \
      : MapImpl(g, r, v) {} \
  \
    DefaultMap(const DefaultMap& copy) \
      : MapImpl(static_cast<const MapImpl&>(copy)) {} \
  \
    template <typename CMap> DefaultMap(const CMap& copy) : MapImpl(copy) {} \
  \
    DefaultMap& operator=(const DefaultMap& copy) { \
      MapImpl::operator=(static_cast<const MapImpl&>(copy)); \
      return *this; \
    } \
  \
    template <typename CMap> DefaultMap& operator=(const CMap& copy) { \
      MapImpl::operator=(copy); \
      return *this; \
    } \
  \
  };


  template <typename MapRegistry, typename Type>
  class DefaultMap : public ArrayMap<MapRegistry, Type> 
  DEFAULT_MAP_BODY(ArrayMap, Type);

  template <typename MapRegistry>
  class DefaultMap<MapRegistry, bool> 
    : public VectorMap<MapRegistry, bool> 
  DEFAULT_MAP_BODY(VectorMap, bool);

  template <typename MapRegistry>
  class DefaultMap<MapRegistry, char> 
    : public VectorMap<MapRegistry, char> 
  DEFAULT_MAP_BODY(VectorMap, char);

  template <typename MapRegistry>
  class DefaultMap<MapRegistry, int> 
    : public VectorMap<MapRegistry, int> 
  DEFAULT_MAP_BODY(VectorMap, int);

  template <typename MapRegistry>
  class DefaultMap<MapRegistry, short> 
    : public VectorMap<MapRegistry, short> 
  DEFAULT_MAP_BODY(VectorMap, short);

  template <typename MapRegistry>
  class DefaultMap<MapRegistry, long> 
    : public VectorMap<MapRegistry, long> 
  DEFAULT_MAP_BODY(VectorMap, long);

  template <typename MapRegistry>
  class DefaultMap<MapRegistry, float> 
    : public VectorMap<MapRegistry, float> 
  DEFAULT_MAP_BODY(VectorMap, float);

  template <typename MapRegistry>
  class DefaultMap<MapRegistry, double> 
    : public VectorMap<MapRegistry, double> 
  DEFAULT_MAP_BODY(VectorMap, double);

  template <typename MapRegistry>
  class DefaultMap<MapRegistry, long double> 
    : public VectorMap<MapRegistry, long double> 
  DEFAULT_MAP_BODY(VectorMap, long double);

  template <typename MapRegistry, typename Type>
  class DefaultMap<MapRegistry, Type*>
    : public VectorMap<MapRegistry, Type*> 
  DEFAULT_MAP_BODY(VectorMap, Type*);

}

#endif
